// ============================================================================
// Copyright (c) 2024 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Mon Apr 15 09:54:27 2024
// ============================================================================

`define ENABLE_DDR4
//`define ENABLE_HPS
//`define ENABLE_HDMI
//`define ENABLE_HSMC
//`define ENABLE_CAM

module golden_top(

      ///////// CLOCK /////////
      input              CLOCK2_50,
      input              CLOCK3_50,
      input              CLOCK_50,

      ///////// KEY /////////
      input              CPU_RESET_N,
      input    [ 3: 0]   KEY,

      ///////// SW /////////
      input    [ 9: 0]   SW,

      ///////// LED /////////
      output   [ 9: 0]   LEDR,

      ///////// Seg7 /////////
      output   [ 6: 0]   HEX0,
      output   [ 6: 0]   HEX1,
      output   [ 6: 0]   HEX2,
      output   [ 6: 0]   HEX3,
      output   [ 6: 0]   HEX4,
      output   [ 6: 0]   HEX5,

      ///////// SDRAM /////////
      output             DRAM_CLK,
      output             DRAM_CKE,
      output   [12: 0]   DRAM_ADDR,
      output   [ 1: 0]   DRAM_BA,
      inout    [15: 0]   DRAM_DQ,
      output             DRAM_LDQM,
      output             DRAM_UDQM,
      output             DRAM_CS_N,
      output             DRAM_WE_N,
      output             DRAM_CAS_N,
      output             DRAM_RAS_N,

`ifdef ENABLE_DDR4
      ///////// DDR4 /////////
      input              DDR4_REFCLK_P,
      output   [16: 0]   DDR4_A,
      output   [ 1: 0]   DDR4_BA,
      output   [ 0: 0]   DDR4_BG,
      output             DDR4_CK,
      output             DDR4_CK_N,
      output             DDR4_CKE,
      inout    [ 3: 0]   DDR4_DQS,
      inout    [ 3: 0]   DDR4_DQS_N,
      inout    [31: 0]   DDR4_DQ,
      inout    [ 3: 0]   DDR4_DBI_N,
      output             DDR4_CS_N,
      output             DDR4_RESET_N,
      output             DDR4_ODT,
      output             DDR4_PAR,
      input              DDR4_ALERT_N,
      output             DDR4_ACT_N,
      input              DDR4_RZQ,
`endif /*ENABLE_DDR4*/

      ///////// Video-In /////////
      input              TD_CLK27,
      input              TD_HS,
      input              TD_VS,
      input    [ 7: 0]   TD_DATA,
      output             TD_RESET_N,

`ifdef ENABLE_HDMI
      ///////// HDMI /////////
      inout              HDMI_LRCLK,
      inout              HDMI_MCLK,
      inout              HDMI_SCLK,
      output             HDMI_TX_CLK,
      output             HDMI_TX_HS,
      output             HDMI_TX_VS,
      output   [23: 0]   HDMI_TX_D,
      output             HDMI_TX_DE,
      input              HDMI_TX_INT,
      inout              HDMI_I2S,
`endif /*ENABLE_HDMI*/

      ///////// Audio /////////
      inout              AUD_BCLK,
      output             AUD_XCK,
      inout              AUD_ADCLRCK,
      input              AUD_ADCDAT,
      inout              AUD_DACLRCK,
      output             AUD_DACDAT,

      ///////// ADC /////////
      output             ADC_SCLK,
      input              ADC_DOUT,
      output             ADC_DIN,
      output             ADC_CONVST,

      ///////// I2C for Audio and Video-In /////////
      output             FPGA_I2C_SCLK,
      inout              FPGA_I2C_SDAT,

      ///////// GPIO /////////
      inout    [35: 0]   GPIO,

`ifdef ENABLE_HSMC
      ///////// HSMC /////////
      output   [ 3: 0]   HSMC_GTS_TX_P,
      output   [ 3: 0]   HSMC_GTS_TX_N,
      input    [ 3: 0]   HSMC_GTS_RX_P,
      input    [ 3: 0]   HSMC_GTS_RX_N,
      input              HSMC_GTS_RX_REFCLK_P,
      input              HSMC_GTS_REFCLK_P,
      inout    [ 1: 0]   HSMC_B5B_D,
// for 013
      //input    [ 2: 0]   HSMC_CLK_IN,
      //inout    [76: 0]   HSMC_D,
`endif /*ENABLE_HSMC*/

`ifdef ENABLE_CAM
      ///////// CAM /////////
      input              CAM_CLK_P,
      input              CAM_CLK_N,
      input    [ 1: 0]   CAM_D_P,
      input    [ 1: 0]   CAM_D_N,
      inout              CAM_I2C_SCL,
      inout              CAM_I2C_SDA,
      inout              CAM_GPIO,
      input              CAM_RZQ1,
`endif /*ENABLE_CAM*/

`ifdef ENABLE_HPS
      ///////// HPS /////////
      input              HPS_CLK_25,
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      inout    [ 1: 0]   HPS_GPIO,
      inout              HPS_GSENSOR_INT,
      inout              HPS_I2C_SCL,
      inout              HPS_I2C_SDA,
      inout              HPS_KEY,
      inout              HPS_LCM_BK,
      inout              HPS_LCM_D_C,
      inout              HPS_LCM_RST_N,
      output             HPS_LCM_SPIM_CLK,
      output             HPS_LCM_SPIM_MOSI,
      output             HPS_LCM_SPIM_SS,
      inout              HPS_LED,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
      input              HPS_USB_CLK,
      inout    [ 7: 0]   HPS_USB_DATA,
      input              HPS_USB_DIR,
      input              HPS_USB_NXT,
      output             HPS_USB_STP,
`endif /*ENABLE_HPS*/

      ///////// IR /////////
      output             IRDA_TXD,
      input              IRDA_RXD

);


//=======================================================
//  REG/WIRE declarations
//=======================================================


wire    [ 9: 0]   led;
wire              HEX0P;
wire              HEX1P;
wire              HEX2P;
wire              HEX3P;
wire              HEX4P;
wire              HEX5P;


//=======================================================
//  Structural coding
//=======================================================

assign LEDR = ~led;

nios_system u0 (
    .clk_clk                        (CLOCK2_50),                    //                     clk.clk
    .reset_reset_n                  (1'b1),                         //                   reset.reset_n
    .sw_external_connection_export  (SW),                           //  sw_external_connection.export
    .led_external_connection_export (led),                          // led_external_connection.export
    .key_external_connection_export (KEY),                          // key_external_connection.export
    .seg7_conduit_end_export        ({HEX5P, HEX5, HEX4P, HEX4,
                                      HEX3P, HEX3, HEX2P, HEX2,
                                      HEX1P, HEX1, HEX0P, HEX0}),   //        seg7_conduit_end.export
    .audio_i2c_export_scl_pad_io    (FPGA_I2C_SCLK),                //        audio_i2c_export.scl_pad_io
    .audio_i2c_export_sda_pad_io    (FPGA_I2C_SDAT),                //                        .sda_pad_io
    .audio_conduit_end_XCK          (AUD_XCK),                      //       audio_conduit_end.XCK
    .audio_conduit_end_ADCDAT       (AUD_ADCDAT),                   //                        .ADCDAT
    .audio_conduit_end_ADCLRC       (AUD_ADCLRCK),                  //                        .ADCLRC
    .audio_conduit_end_DACDAT       (AUD_DACDAT),                   //                        .DACDAT
    .audio_conduit_end_DACLRC       (AUD_DACLRCK),                  //                        .DACLRC
    .audio_conduit_end_BCLK         (AUD_BCLK),                     //                        .BCLK

    .emif_ref_clk_clk               (DDR4_REFCLK_P),                //            emif_ref_clk.clk
    .emif_mem_mem_ck_t              (DDR4_CK),                      //                emif_mem.mem_ck_t
    .emif_mem_mem_ck_c              (DDR4_CK_N),                    //                        .mem_ck_c
    .emif_mem_mem_cke               (DDR4_CKE),                     //                        .mem_cke
    .emif_mem_mem_odt               (DDR4_ODT),                     //                        .mem_odt
    .emif_mem_mem_cs_n              (DDR4_CS_N),                    //                        .mem_cs_n
    .emif_mem_mem_a                 (DDR4_A),                       //                        .mem_a
    .emif_mem_mem_ba                (DDR4_BA),                      //                        .mem_ba
    .emif_mem_mem_bg                (DDR4_BG),                      //                        .mem_bg
    .emif_mem_mem_act_n             (DDR4_ACT_N),                   //                        .mem_act_n
    .emif_mem_mem_par               (DDR4_PAR),                     //                        .mem_par
    .emif_mem_mem_alert_n           (DDR4_ALERT_N),                 //                        .mem_alert_n
    .emif_mem_mem_reset_n           (DDR4_RESET_N),                 //                        .mem_reset_n
    .emif_mem_mem_dq                (DDR4_DQ),                      //                        .mem_dq
    .emif_mem_mem_dqs_t             (DDR4_DQS),                     //                        .mem_dqs_t
    .emif_mem_mem_dqs_c             (DDR4_DQS_N),                   //                        .mem_dqs_c
    .emif_mem_mem_dbi_n             (DDR4_DBI_N),                   //                        .mem_dbi_n
    .emif_oct_oct_rzqin             (DDR4_RZQ)                      //                emif_oct.oct_rzqin

);




endmodule
