module BOARD_INFO (	

   input            CLK_50,	
   input            RESET_N, 
   output           I2C_SCL, 
   inout            I2C_SDA,
	output           CLK_400,///clk 400k

   output [15:0] Board_Version  ,
   output [15:0] MaxCode_Version,
   output [15:0] PowerIn_Voltage,
   output [15:0] Fan_Speed      ,
   output [15:0] Temp_FPGA      ,
   output [15:0] Temp_Board     ,
   output [15:0] PIN_STATUS     ,
   output [15:0] Temp_Board2    ,
   output [15:0] Shutdown_flag  ,
   output [15:0] PG_ErrorMask   ,
   output	reg       Info_Valid ,    
		
	//--TEST -- 
	output reg [7:0]   BYTE ,
	output reg [7:0]   SLAVE_ADDR , 
   output reg [7:0]   ST ,
   output reg [7:0]   CNT,
   output reg [7:0]   W_POINTER_REG,
	output             W_POINTER_END,
   output reg         W_POINTER_GO,
   output             R_END,
   output reg         R_GO,
   output     [15:0]  R_DATA ,
	output             SDAO
				);
					

//=======================================================
//  parameter 
//=======================================================
parameter [7:0] P_Board_Version_H   =8'd0 ;
parameter [7:0] P_Board_Version_L   =8'd1 ;
parameter [7:0] P_MaxCode_Version_H =8'd2 ;
parameter [7:0] P_MaxCode_Version_L =8'd3 ;
parameter [7:0] P_PowerIn_Voltage_H =8'd4 ;
parameter [7:0] P_PowerIn_Voltage_L =8'd5 ;
parameter [7:0] P_Fan_Speed_H       =8'd6 ;
parameter [7:0] P_Fan_Speed_L       =8'd7;
parameter [7:0] P_Temp_FPGA_H       =8'd8;
parameter [7:0] P_Temp_FPGA_L       =8'd9;
parameter [7:0] P_Temp_Board_H      =8'd10;
parameter [7:0] P_Temp_Board_L      =8'd11;
parameter [7:0] P_PIN_STATUS_H     =8'd12;
parameter [7:0] P_PIN_STATUS_L     =8'd13;
parameter [7:0] P_Temp_Board2_H     =8'd14;
parameter [7:0] P_Temp_Board2_L     =8'd15;
parameter [7:0] P_Shutdown_flag_H   =8'd16;
parameter [7:0] P_Shutdown_flag_L   =8'd17;
parameter [7:0] P_PG_ErrorMask_H    =8'd18;
parameter [7:0] P_PG_ErrorMask_L    =8'd19;



//---max10 mapping                                      
//.R00       (Board_Version  [15:8]),
//.R01       (Board_Version  [7 :0]),
//.R02       (MaxCode_Version[15:8]),
//.R03       (MaxCode_Version[7 :0]),
//.R04       (PowerIn_Voltage[15:8]),
//.R05       (PowerIn_Voltage[7 :0]),
//.R06       (Fan_Speed      [15:8]),
//.R07       (Fan_Speed      [7 :0]),
//.R08       (Temp_FPGA      [15:8]),
//.R09       (Temp_FPGA      [7 :0]),
//.R10       (Temp_Board     [15:8]),
//.R11       (Temp_Board     [7 :0]),
//.R12       (PIN_STATUS     [15:8]),
//.R13       (PIN_STATUS     [7 :0]),
//.R14       (Temp_Board2    [15:8]),
//.R15       (Temp_Board2    [7 :0]),
//.R16       (Shutdown_flag  [15:8]),
//.R17       (Shutdown_flag  [7 :0]),
//.R18       (PG_ErrorMask   [15:8]),
//.R19       (PG_ErrorMask   [7 :0])			


//=======================================================
//  WIRE /REGISETER 
//=======================================================
reg [7:0] Board_Version_H ; 
reg [7:0] Board_Version_L  ;
reg [7:0] MaxCode_Version_H;
reg [7:0] MaxCode_Version_L;
reg [7:0] PowerIn_Voltage_H;
reg [7:0] PowerIn_Voltage_L;
reg [7:0] Fan_Speed_H      ;
reg [7:0] Fan_Speed_L      ;
reg [7:0] Temp_FPGA_H      ;
reg [7:0] Temp_FPGA_L      ;
reg [7:0] Temp_Board_H     ;
reg [7:0] Temp_Board_L     ;
reg [7:0] PIN_STATUS_H    ;
reg [7:0] PIN_STATUS_L    ;
reg [7:0] Temp_Board2_H    ;
reg [7:0] Temp_Board2_L    ;
reg [7:0] Shutdown_flag_H  ;
reg [7:0] Shutdown_flag_L  ;
reg [7:0] PG_ErrorMask_H   ;
reg [7:0] PG_ErrorMask_L   ;

assign Board_Version   ={  Board_Version_H ,Board_Version_L   };
assign MaxCode_Version ={  MaxCode_Version_H,MaxCode_Version_L};
assign PowerIn_Voltage ={  PowerIn_Voltage_H,PowerIn_Voltage_L};
assign Fan_Speed       ={  Fan_Speed_H      ,Fan_Speed_L      };
assign Temp_FPGA       ={  Temp_FPGA_H      ,Temp_FPGA_L      };
assign Temp_Board      ={  Temp_Board_H     ,Temp_Board_L     };
assign PIN_STATUS      ={  PIN_STATUS_H    ,PIN_STATUS_L    };
assign Temp_Board2     ={  Temp_Board2_H    ,Temp_Board2_L    };
assign Shutdown_flag   ={  Shutdown_flag_H  ,Shutdown_flag_L  };
assign PG_ErrorMask    ={  PG_ErrorMask_H   ,PG_ErrorMask_L   };

//--I2C Main Controller--- 
reg  [31:0] DELY;

//-----I2C-BUS-I/O----
wire        W_POINTER_SCL; 
wire        W_POINTER_SDAO; 
wire        R_SCL; 
wire        R_SDAO;  
//=======================================================
//  Parameter  
//=======================================================

// I2C R NUMBER 
parameter    READ_MAX       = 20; 
    
//slave address   
parameter    SLAVE_ADDR_F   = 8'h8c;

//registers address


//------------------------------------
//=======================================================
//  Structural coding
//======================================================= 
																 
CLOCKMEM CLOCKMEM_inst2(  .RESET_n(1), .CLK    (CLK_50),.CLK_FREQ(125) ,.CK_1HZ (CLK_400)) ;

//--I2C Main Controller--- 
always @(negedge RESET_N or posedge CLK_400 )begin 
if (!RESET_N  ) begin 
    ST <=0;
 	 W_POINTER_GO <=1;
    R_GO  <=1 ;		 
	 CNT  <=0;
	 DELY <=0 ;
	 Info_Valid <=0;
	 
end 
else begin
case (ST)
0: begin 
    ST<=1; //---First ,  to do I2C read . 
	 W_POINTER_GO <=1;
    R_GO         <=1 ;		 
	 CNT          <=0;
	 DELY         <=0 ;   
   end
//------------- READ -------	
1: begin 
    ST<=2; 

	 Board_Version_H  <=0;
	 Board_Version_L  <=0;
	 MaxCode_Version_H<=0;
	 MaxCode_Version_L<=0;
	 PowerIn_Voltage_H<=0;
	 PowerIn_Voltage_L<=0;
	 Fan_Speed_H      <=0;
	 Fan_Speed_L      <=0;
	 Temp_FPGA_H      <=0;
	 Temp_FPGA_L      <=0;
	 Temp_Board_H     <=0;
	 Temp_Board_L     <=0;
	 PIN_STATUS_H    <=0;
	 PIN_STATUS_L    <=0;
	 Temp_Board2_H    <=0;
	 Temp_Board2_L    <=0;
	 Shutdown_flag_H  <=0;
	 Shutdown_flag_L  <=0;
	 PG_ErrorMask_H   <=0;
	 PG_ErrorMask_L   <=0;
	end	
2: begin
        //  READ 
	          if ( CNT==0)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Board_Version_H    ,8'h1 };
		  else if ( CNT==1)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Board_Version_L    ,8'h1 };
		  else if ( CNT==2)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_MaxCode_Version_H  ,8'h1 };
		  else if ( CNT==3)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_MaxCode_Version_L  ,8'h1 };
		  else if ( CNT==4)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_PowerIn_Voltage_H  ,8'h1 };
		  else if ( CNT==5)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_PowerIn_Voltage_L  ,8'h1 };
		  else if ( CNT==6)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Fan_Speed_H        ,8'h1 };
		  else if ( CNT==7)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Fan_Speed_L        ,8'h1 };
		  else if ( CNT==8)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Temp_FPGA_H        ,8'h1 };
		  else if ( CNT==9)   {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Temp_FPGA_L        ,8'h1 };
		  else if ( CNT==10)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Temp_Board_H       ,8'h1 };
		  else if ( CNT==11)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Temp_Board_L       ,8'h1 };
		  else if ( CNT==12)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_PIN_STATUS_H       ,8'h1 };
		  else if ( CNT==13)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_PIN_STATUS_L       ,8'h1 };
		  else if ( CNT==14)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Temp_Board2_H      ,8'h1 };
		  else if ( CNT==15)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Temp_Board2_L      ,8'h1 };
		  else if ( CNT==16)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Shutdown_flag_H    ,8'h1 };
		  else if ( CNT==17)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_Shutdown_flag_L    ,8'h1 };
		  else if ( CNT==18)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_PG_ErrorMask_H     ,8'h1 };
		  else if ( CNT==19)  {SLAVE_ADDR[7:0] ,W_POINTER_REG,BYTE}<= {SLAVE_ADDR_F[7:0] ,P_PG_ErrorMask_L     ,8'h1 };
		                
	if ( W_POINTER_END ) begin  
	   W_POINTER_GO  <=0; 
		ST  <=3 ; 
		DELY<=0;  
	 end
	end                
	//------- Write Pointer
3: begin 
    DELY  <=DELY +1;
    if ( DELY ==2 ) begin 
      W_POINTER_GO  <=1;
      ST            <=4 ; 
	 end
	end       
4: begin 
    if  ( W_POINTER_END ) ST<=5 ; 	
	end              
5: begin 
    ST<=6 ; 
	end 
	//------- Read DATA  		 
6: begin 
	 if ( R_END ) begin  
	  R_GO  <=0; 
	  ST<=7 ; 
	  DELY<=0; 
	 end
	end                
7: begin 
    DELY  <=DELY +1;
    if ( DELY ==2 ) begin 	 
      R_GO  <=1;
      ST<=8 ; 
	 end
	end       
8: begin 
     ST<=9 ; 
	end       
9: begin 
   if  ( R_END ) 
	 begin 		
	      //---FAN READ  
		        if ( CNT==0)  	Board_Version_H   =R_DATA[7:0] ; 
			else if ( CNT==1)  	Board_Version_L   =R_DATA[7:0] ; 				  
		   else if ( CNT==2)  	MaxCode_Version_H =R_DATA[7:0] ;
		   else if ( CNT==3)  	MaxCode_Version_L =R_DATA[7:0] ;
		   else if ( CNT==4)  	PowerIn_Voltage_H = R_DATA[7:0];
		   else if ( CNT==5)  	PowerIn_Voltage_L = R_DATA[7:0];
		   else if ( CNT==6)  	Fan_Speed_H       = R_DATA[7:0];
		   else if ( CNT==7)  	Fan_Speed_L       = R_DATA[7:0];
		   else if ( CNT==8)  	Temp_FPGA_H       = R_DATA[7:0];
		   else if ( CNT==9)  	Temp_FPGA_L       = R_DATA[7:0];
		   else if ( CNT==10) 	Temp_Board_H      = R_DATA[7:0];
		   else if ( CNT==11) 	Temp_Board_L      = R_DATA[7:0];
		   else if ( CNT==12) 	PIN_STATUS_H      = R_DATA[7:0];
		   else if ( CNT==13) 	PIN_STATUS_L      = R_DATA[7:0];
		   else if ( CNT==14) 	Temp_Board2_H     = R_DATA[7:0];
		   else if ( CNT==15) 	Temp_Board2_L     = R_DATA[7:0];
		   else if ( CNT==16) 	Shutdown_flag_H   = R_DATA[7:0];
		   else if ( CNT==17) 	Shutdown_flag_L   = R_DATA[7:0];
		   else if ( CNT==18) 	PG_ErrorMask_H    = R_DATA[7:0];
		   else if ( CNT==19) 	PG_ErrorMask_L    = R_DATA[7:0];
	      CNT<=CNT+1 ;
	      ST<=10 ; 	
	 end 
  end	
10: begin   
	     W_POINTER_GO <=1;
        R_GO         <=1 ;		 		    		 		  
 //-----------check read end 
     if ( CNT == READ_MAX)  begin 
	     CNT          <= 0 ;   
	     ST           <=11; 
		  DELY         <=0;
		  Info_Valid   <=1;
	  end	  
	  else 	  ST           <=2; 
end 
11: begin	 
    DELY  <=DELY +1;
    if ( DELY ==400 ) //1ms delay 
	 begin 
      ST<=2 ;
	   Info_Valid   <=0;	
	 end
 
end	 
//----  READ END ----
endcase
end
end
//-----------------------------MAIN-ST END ------------------------------------------

//============inout  i2c  q15(a10) example ===============  
// wire const_zero_sig /* synthesis keep */;
// assign const_zero_sig = 1\'b0;
// assign TRI_PIN = enable? const_zero_sig : \'bz;
//========================================================
wire   const_zero_sig/* synthesis keep */ ; 
assign const_zero_sig = 0 ; 
assign I2C_SDA        = (SDAO)?1'bz :const_zero_sig;
assign I2C_SCL        = W_POINTER_SCL  & R_SCL   ;
assign SDAO           = W_POINTER_SDAO & R_SDAO  ;


//==== I2C WRITE POINTER ===
I2C_WRITE_POINTER  I2C_WRITE_POINTER_i(
   .RESET_N      ( RESET_N   ),
	.PT_CK        ( CLK_400 ),
	.GO           ( W_POINTER_GO ),
	.POINTER      ( W_POINTER_REG),
	.SLAVE_ADDRESS( SLAVE_ADDR ),
	.SDAI         ( I2C_SDA    ),
	.SDAO         ( W_POINTER_SDAO),
	.SCLO         ( W_POINTER_SCL ),
	.END_OK       ( W_POINTER_END )
);
//-----I2C TO READ---- 
I2C_READ_DATA I2C_READ_DATA_i( 
   .RESET_N      ( RESET_N    ),
	.PT_CK        ( CLK_400   ),
	.GO           ( R_GO       ),
	.SLAVE_ADDRESS( SLAVE_ADDR ),
 	.BYTE_NUM     ( BYTE       ) , // I2C read 8 bit data 
	.SDAI         ( I2C_SDA),
	.SDAO         ( R_SDAO ),
	.SCLO         ( R_SCL  ),
	.END_OK       ( R_END  ),
	.DATA         ( R_DATA )
);

endmodule
