// ============================================================================
// Copyright (c) 2024 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Mon Apr 15 09:54:27 2024
// ============================================================================

//`define ENABLE_DDR4
//`define ENABLE_HPS
//`define ENABLE_HDMI
//`define ENABLE_HSMC
//`define ENABLE_CAM

module golden_top(

      ///////// CLOCK /////////
      input              CLOCK2_50,
      input              CLOCK3_50,
      input              CLOCK_50,

      ///////// KEY /////////
      input              CPU_RESET_N,
      input    [ 3: 0]   KEY,

      ///////// SW /////////
      input    [ 9: 0]   SW,

      ///////// LED /////////
      output   [ 9: 0]   LEDR,

      ///////// Seg7 /////////
      output   [ 6: 0]   HEX0,
      output   [ 6: 0]   HEX1,
      output   [ 6: 0]   HEX2,
      output   [ 6: 0]   HEX3,
      output   [ 6: 0]   HEX4,
      output   [ 6: 0]   HEX5,

      ///////// SDRAM /////////
      output             DRAM_CLK,
      output             DRAM_CKE,
      output   [12: 0]   DRAM_ADDR,
      output   [ 1: 0]   DRAM_BA,
      inout    [15: 0]   DRAM_DQ,
      output             DRAM_LDQM,
      output             DRAM_UDQM,
      output             DRAM_CS_N,
      output             DRAM_WE_N,
      output             DRAM_CAS_N,
      output             DRAM_RAS_N,

`ifdef ENABLE_DDR4
      ///////// DDR4 /////////
      input              DDR4_REFCLK_P,
      output   [16: 0]   DDR4_A,
      output   [ 1: 0]   DDR4_BA,
      output   [ 0: 0]   DDR4_BG,
      output             DDR4_CK,
      output             DDR4_CK_N,
      output             DDR4_CKE,
      inout    [ 3: 0]   DDR4_DQS,
      inout    [ 3: 0]   DDR4_DQS_N,
      inout    [31: 0]   DDR4_DQ,
      inout    [ 3: 0]   DDR4_DBI_N,
      output             DDR4_CS_N,
      output             DDR4_RESET_N,
      output             DDR4_ODT,
      output             DDR4_PAR,
      input              DDR4_ALERT_N,
      output             DDR4_ACT_N,
      input              DDR4_RZQ,
`endif /*ENABLE_DDR4*/

      ///////// Video-In /////////
      input              TD_CLK27,
      input              TD_HS,
      input              TD_VS,
      input    [ 7: 0]   TD_DATA,
      output             TD_RESET_N,

`ifdef ENABLE_HDMI
      ///////// HDMI /////////
      inout              HDMI_LRCLK,
      inout              HDMI_MCLK,
      inout              HDMI_SCLK,
      output             HDMI_TX_CLK,
      output             HDMI_TX_HS,
      output             HDMI_TX_VS,
      output   [23: 0]   HDMI_TX_D,
      output             HDMI_TX_DE,
      input              HDMI_TX_INT,
      inout              HDMI_I2S,
`endif /*ENABLE_HDMI*/

      ///////// Audio /////////
      inout              AUD_BCLK,
      output             AUD_XCK,
      inout              AUD_ADCLRCK,
      input              AUD_ADCDAT,
      inout              AUD_DACLRCK,
      output             AUD_DACDAT,

      ///////// ADC /////////
      output             ADC_SCLK,
      input              ADC_DOUT,
      output             ADC_DIN,
      output             ADC_CONVST,

      ///////// I2C for Audio and Video-In /////////
      output             FPGA_I2C_SCLK,
      inout              FPGA_I2C_SDAT,

      ///////// GPIO /////////
      inout    [35: 0]   GPIO,

`ifdef ENABLE_HSMC
      ///////// HSMC /////////
      output   [ 3: 0]   HSMC_GTS_TX_P,
      output   [ 3: 0]   HSMC_GTS_TX_N,
      input    [ 3: 0]   HSMC_GTS_RX_P,
      input    [ 3: 0]   HSMC_GTS_RX_N,
      input              HSMC_GTS_RX_REFCLK_P,
      input              HSMC_GTS_REFCLK_P,
      inout    [ 1: 0]   HSMC_B5B_D,
// for 013
      //input    [ 2: 0]   HSMC_CLK_IN,
      //inout    [76: 0]   HSMC_D,
`endif /*ENABLE_HSMC*/

`ifdef ENABLE_CAM
      ///////// CAM /////////
      input              CAM_CLK_P,
      input              CAM_CLK_N,
      input    [ 1: 0]   CAM_D_P,
      input    [ 1: 0]   CAM_D_N,
      inout              CAM_I2C_SCL,
      inout              CAM_I2C_SDA,
      inout              CAM_GPIO,
      input              CAM_RZQ1,
`endif /*ENABLE_CAM*/

`ifdef ENABLE_HPS
      ///////// HPS /////////
      input              HPS_CLK_25,
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      inout    [ 1: 0]   HPS_GPIO,
      inout              HPS_GSENSOR_INT,
      inout              HPS_I2C_SCL,
      inout              HPS_I2C_SDA,
      inout              HPS_KEY,
      inout              HPS_LCM_BK,
      inout              HPS_LCM_D_C,
      inout              HPS_LCM_RST_N,
      output             HPS_LCM_SPIM_CLK,
      output             HPS_LCM_SPIM_MOSI,
      output             HPS_LCM_SPIM_SS,
      inout              HPS_LED,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
      input              HPS_USB_CLK,
      inout    [ 7: 0]   HPS_USB_DATA,
      input              HPS_USB_DIR,
      input              HPS_USB_NXT,
      output             HPS_USB_STP,
`endif /*ENABLE_HPS*/

      ///////// IR /////////
      output             IRDA_TXD,
      input              IRDA_RXD

);

// LED set
assign LEDR[9:1] = {9'hff}; 

// Seg7 off
assign  HEX0=7'h7f; 
assign  HEX1=7'h7f; 
assign  HEX2=7'h7f; 
assign  HEX3=7'h7f; 
assign  HEX4=7'h7f; 
assign  HEX5=7'h7f; 

//=======================================================
//  REG/WIRE declarations
//=======================================================
//System Monitor IP
//IP interface
wire        Info_Valid;
wire [15:0] Board_Version	;//A~F.ex BOARD_VERSION = 16'h000b ,express to revB
wire [15:0] MaxCode_Version;//0~F.ex MAX10_VERSION = 16'h0004 ,express to rev4
wire [15:0] PowerIn_Voltage;//unit : Celsius degree    
wire [15:0] Fan_Speed	   ;//unit : rpm  
wire [15:0] Temp_FPGA	   ;//unit : Celsius degree 
wire [15:0] Temp_Board	   ;//unit : Celsius degree 
wire [15:0] PIN_STATUS	   ;//
wire [15:0] Temp_Board2	   ;//unit : Celsius degree 
wire [15:0] Shutdown_flag  ;//unit : Celsius degree
wire        CLK_400 ; 

//----reg
reg [15:0] rBoard_Version	;
reg [15:0] rMaxCode_Version;
reg [15:0] rPowerIn_Voltage;
reg [15:0] rFan_Speed	   ;
reg [15:0] rTemp_FPGA	   ; 
reg [15:0] rTemp_Board	   ; 
reg [15:0] rPIN_STATUS	   ; 
reg [15:0] rTemp_Board2	   ; 
reg [15:0] rShutdown_flag  ;

//=======================================================
//  Structural coding
//=======================================================

//---test----------------------------
assign LEDR[0]=   CLK_400 ;             

//----------BOARD_INFO by I2C BUS --- 
BOARD_INFO BOARD_INFO_i(    
             .RESET_N          (KEY[0]     ),  // input
             .CLK_50           (CLOCK_50   ),  // input
				 .CLK_400          (CLK_400    ),  // output 
             // 2-pin I2C master interface
             .I2C_SCL          (FPGA_I2C_SCLK),
             .I2C_SDA          (FPGA_I2C_SDAT),
             // status report (output)
             .Info_Valid       ( Info_Valid     ), //1: following data is valid
             .Board_Version	 ( Board_Version	), //A~F.ex BOARD_VERSION = 16'h000b ,express to revB
             .MaxCode_Version  ( MaxCode_Version), //0~F.ex MAX10_VERSION = 16'h0004 ,express to rev4
             .PowerIn_Voltage  ( PowerIn_Voltage), //unit : Celsius degree    
				 .Fan_Speed	       ( Fan_Speed	   ), //unit : rpm                                     
             .Temp_FPGA	       ( Temp_FPGA	   ), //unit : Celsius degree 
				 .Temp_Board	    ( Temp_Board	   ), //unit : Celsius degree
				 .Temp_Board2      ( Temp_Board2    ), //unit : Celsius degree
				 .PIN_STATUS	    ( PIN_STATUS	   ), //unit : bit
             .Shutdown_flag	 ( Shutdown_flag  )	//unit : bit			 
				 
);

always @(posedge    CLK_400 ) begin
    if (Info_Valid) begin
        rBoard_Version	<= Board_Version	;
        rMaxCode_Version<= MaxCode_Version;
        rPowerIn_Voltage<= PowerIn_Voltage;
        rFan_Speed	   <= Fan_Speed	   ;
		  rTemp_FPGA	   <= Temp_FPGA	   ;
		  rTemp_Board	   <= Temp_Board	   ;
		  rPIN_STATUS	   <= PIN_STATUS	   ;
		  rTemp_Board2	   <= Temp_Board2	   ;
		  rShutdown_flag  <= Shutdown_flag  ;
    end
end

endmodule