// ============================================================================
// Copyright (c) 2024 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Mon Apr 15 09:54:27 2024
// ============================================================================

//`define ENABLE_DDR4
//`define ENABLE_HPS
`define ENABLE_HDMI
//`define ENABLE_HSMC
//`define ENABLE_CAM

module golden_top(

      ///////// CLOCK /////////
      input              CLOCK2_50,
      input              CLOCK3_50,
      input              CLOCK_50,

      ///////// KEY /////////
      input              CPU_RESET_N,
      input    [ 3: 0]   KEY,

      ///////// SW /////////
      input    [ 9: 0]   SW,

      ///////// LED /////////
      output   [ 9: 0]   LEDR,

      ///////// Seg7 /////////
      output   [ 6: 0]   HEX0,
      output   [ 6: 0]   HEX1,
      output   [ 6: 0]   HEX2,
      output   [ 6: 0]   HEX3,
      output   [ 6: 0]   HEX4,
      output   [ 6: 0]   HEX5,

      ///////// SDRAM /////////
      output             DRAM_CLK,
      output             DRAM_CKE,
      output   [12: 0]   DRAM_ADDR,
      output   [ 1: 0]   DRAM_BA,
      inout    [15: 0]   DRAM_DQ,
      output             DRAM_LDQM,
      output             DRAM_UDQM,
      output             DRAM_CS_N,
      output             DRAM_WE_N,
      output             DRAM_CAS_N,
      output             DRAM_RAS_N,

`ifdef ENABLE_DDR4
      ///////// DDR4 /////////
      input              DDR4_REFCLK_P,
      output   [16: 0]   DDR4_A,
      output   [ 1: 0]   DDR4_BA,
      output   [ 0: 0]   DDR4_BG,
      output             DDR4_CK,
      output             DDR4_CK_N,
      output             DDR4_CKE,
      inout    [ 3: 0]   DDR4_DQS,
      inout    [ 3: 0]   DDR4_DQS_N,
      inout    [31: 0]   DDR4_DQ,
      inout    [ 3: 0]   DDR4_DBI_N,
      output             DDR4_CS_N,
      output             DDR4_RESET_N,
      output             DDR4_ODT,
      output             DDR4_PAR,
      input              DDR4_ALERT_N,
      output             DDR4_ACT_N,
      input              DDR4_RZQ,
`endif /*ENABLE_DDR4*/

      ///////// Video-In /////////
      input              TD_CLK27,
      input              TD_HS,
      input              TD_VS,
      input    [ 7: 0]   TD_DATA,
      output             TD_RESET_N,

`ifdef ENABLE_HDMI
      ///////// HDMI /////////
      inout              HDMI_LRCLK,
      inout              HDMI_MCLK,
      inout              HDMI_SCLK,
      output             HDMI_TX_CLK,
      output             HDMI_TX_HS,
      output             HDMI_TX_VS,
      output   [23: 0]   HDMI_TX_D,
      output             HDMI_TX_DE,
      input              HDMI_TX_INT,
      inout              HDMI_I2S,
`endif /*ENABLE_HDMI*/

      ///////// Audio /////////
      inout              AUD_BCLK,
      output             AUD_XCK,
      inout              AUD_ADCLRCK,
      input              AUD_ADCDAT,
      inout              AUD_DACLRCK,
      output             AUD_DACDAT,

      ///////// ADC /////////
      output             ADC_SCLK,
      input              ADC_DOUT,
      output             ADC_DIN,
      output             ADC_CONVST,

      ///////// I2C for Audio and Video-In /////////
      output             FPGA_I2C_SCLK,
      inout              FPGA_I2C_SDAT,

      ///////// GPIO /////////
      inout    [35: 0]   GPIO,

`ifdef ENABLE_HSMC
      ///////// HSMC /////////
      output   [ 3: 0]   HSMC_GTS_TX_P,
      output   [ 3: 0]   HSMC_GTS_TX_N,
      input    [ 3: 0]   HSMC_GTS_RX_P,
      input    [ 3: 0]   HSMC_GTS_RX_N,
      input              HSMC_GTS_RX_REFCLK_P,
      input              HSMC_GTS_REFCLK_P,
      inout    [ 1: 0]   HSMC_B5B_D,
// for 013
      //input    [ 2: 0]   HSMC_CLK_IN,
      //inout    [76: 0]   HSMC_D,
`endif /*ENABLE_HSMC*/

`ifdef ENABLE_CAM
      ///////// CAM /////////
      input              CAM_CLK_P,
      input              CAM_CLK_N,
      input    [ 1: 0]   CAM_D_P,
      input    [ 1: 0]   CAM_D_N,
      inout              CAM_I2C_SCL,
      inout              CAM_I2C_SDA,
      inout              CAM_GPIO,
      input              CAM_RZQ1,
`endif /*ENABLE_CAM*/

`ifdef ENABLE_HPS
      ///////// HPS /////////
      input              HPS_CLK_25,
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      inout    [ 1: 0]   HPS_GPIO,
      inout              HPS_GSENSOR_INT,
      inout              HPS_I2C_SCL,
      inout              HPS_I2C_SDA,
      inout              HPS_KEY,
      inout              HPS_LCM_BK,
      inout              HPS_LCM_D_C,
      inout              HPS_LCM_RST_N,
      output             HPS_LCM_SPIM_CLK,
      output             HPS_LCM_SPIM_MOSI,
      output             HPS_LCM_SPIM_SS,
      inout              HPS_LED,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
      input              HPS_USB_CLK,
      inout    [ 7: 0]   HPS_USB_DATA,
      input              HPS_USB_DIR,
      input              HPS_USB_NXT,
      output             HPS_USB_STP,
`endif /*ENABLE_HPS*/

      ///////// IR /////////
      output             IRDA_TXD,
      input              IRDA_RXD

);


//=======================================================
//  REG/WIRE declarations
//=======================================================




//=======================================================
//  Structural coding
//=======================================================




assign reset_n = !ninit_done & CPU_RESET_N;
wire ninit_done;
agilex_reset_release reset_release_Inst (
	.ninit_done(ninit_done)
	);
	

	
wire beat;
heart_beat heart_beat_clk50(
	.clk(CLOCK2_50),
	.led(beat)
	);


assign LEDR = (!KEY[0])?10'h0:
				  (!KEY[1])?10'h3ff:{~PWM_width[6],~PWM_width[6],~PWM_width[6],~PWM_width[6],~PWM_width[6],
										    PWM_width[6], PWM_width[6], PWM_width[6], PWM_width[6], PWM_width[6]};
								
wire [23:0]	mSEG7_DIG;		
assign	mSEG7_DIG	=	KEY[0]? {	counter[27:24], counter[27:24], counter[27:24], counter[27:24], counter[27:24], counter[27:24]} :{6{4'b1000}};				 

//7 segment LUT
SEG7_LUT_6 SEG7_LUT_6_Inst 
(	
	.oSEG0(HEX0),
	.oSEG1(HEX1),
	.oSEG2(HEX2),
	.oSEG3(HEX3),
	.oSEG4(HEX4),
	.oSEG5(HEX5),
	.iDIG(mSEG7_DIG) 
);
								

reg 	[27:0] 	counter;
reg 	[5:0]   PWM_adj;
reg   [6:0]   PWM_width;
always @(posedge CLOCK2_50 or negedge KEY[2])
begin
	if(!KEY[2])
		begin
			counter <= 0;
		end
	else 
	begin
		counter   <= counter+1;
		PWM_width <= PWM_width[5:0]+ PWM_adj;
		if(counter[26])
		begin
			PWM_adj <= counter[25:20];
		end
		else 
		begin
			PWM_adj <= ~ counter[25:20];
		end
	end
end



pll pll_Inst (
    .refclk   (CLOCK2_50),   //   input,  width = 1,  refclk.clk
    .locked   (),   //  output,  width = 1,  locked.export
    .rst      (~reset_n),      //   input,  width = 1,   reset.reset
    .outclk_0 (HDMI_TX_CLK)  //  output,  width = 1, outclk0.clk
);


	 
//HDMI I2C	
I2C_HDMI_Config u_I2C_HDMI_Config (
	.iCLK(CLOCK2_50),
	.iRST_N(reset_n),
	.I2C_SCLK(FPGA_I2C_SCLK),
	.I2C_SDAT(FPGA_I2C_SDAT),
	.HDMI_TX_INT(HDMI_TX_INT)
	);

vga_controller vga_ins(.iRST_n(reset_n),
                      .iVGA_CLK(HDMI_TX_CLK),
                      .oBLANK_n(HDMI_TX_DE),
                      .oHS(HDMI_TX_HS),
                      .oVS(HDMI_TX_VS),
                      .b_data(HDMI_TX_D[7:0]),
                      .g_data(HDMI_TX_D[15:8]),
                      .r_data(HDMI_TX_D[23:16]));	
	
	



endmodule
